#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "grid.h"
#include "xgroove.h"

int
grid_move (Grid *grid, int from_h, int from_v, int to_h, int to_v)
{
  return 0;
}

int
grid_switch (Grid *grid, GridLoc *from, GridLoc *to)
{
  return 0;
}


Grid *
init_grid ()
{
  Grid   *grid;
  int    i, j;

  if (NULL == (grid = malloc (sizeof(struct _Grid))) ) {
    fprintf (stderr, "init_grid: failed to malloc() [1]\n");
    return NULL;
  }

  grid->grid = NULL;

  if (NULL == (grid->grid = malloc (sizeof(GridLoc *) * GRID_SIZE_ROW)) ) {
    fprintf (stderr, "init_grid: failed in malloc() [2]\n");
    deinit_grid (grid);
    return NULL;
  }

  bzero (grid->grid, sizeof(GridLoc *) * GRID_SIZE_ROW);

  for (i = 0; i < GRID_SIZE_ROW; i++) {
    if (NULL == (grid->grid[i] = malloc (sizeof(GridLoc *) * GRID_SIZE_COL)) ) {
      printf ("init_grid: failed in malloc() [3]\n");
      deinit_grid (grid);
      return NULL;
    }

    for (j = 0; j < GRID_SIZE_COL; j++)
      grid->grid[i][j] = 0;
  }

  grid->count = 0;
  grid->height = GRID_SIZE_ROW;
  grid->width = GRID_SIZE_COL;

  return grid;
}

void
deinit_grid (Grid *grid)
{
  int     i;

  if (grid->grid != NULL) {
    for (i = 0; i < GRID_SIZE_ROW; i++)
      if (grid->grid[i] != 0)
        free (grid->grid[i]);

    free (grid->grid);
  }

  free (grid);
}

GridLoc *
init_gridloc ()
{
  GridLoc     *gloc;

  if (NULL == (gloc = malloc (sizeof(struct _GridLoc))) ) {
    fprintf (stderr, "init_gridloc: failed in malloc() [1]\n");
    return NULL;
  }

  gloc->pos_h = -1;
  gloc->pos_v = -1;
  gloc->name  = NULL;

  return gloc;
}

void
deinit_gridloc (GridLoc *gloc)
{
  if (gloc == NULL)
    return;

  free (gloc);
}

int
grid_add (Grid *grid, GridLoc *gloc)
{
  int     i, j;

  for (i = 0; i < GRID_SIZE_ROW; i++)
    for (j = 0; j < GRID_SIZE_COL; j++)
      if (grid->grid[i][j] == 0) {
        grid->grid[i][j] = (int)gloc;
	gloc->pos_v = i;
	gloc->pos_h = j;
        printf ("grid_add: new gridloc placed at: %d,%d\n", gloc->pos_h, gloc->pos_v);
	return 0;
      }

  fprintf (stderr, "failed to place %s; perhaps the grid is full?\n", gloc->name);

  return -1;
}

void
gridloc_setname (GridLoc *gloc, char *name)
{
  if (NULL == (gloc->name = realloc (gloc->name, strlen(name) + 1)) ) {
    fprintf (stderr, "gridloc_setname: failed in malloc() [1]\n");
    return;
  }

  strncpy (gloc->name, name, strlen(name) + 1);

  printf ("gridloc_setname: set new name for %d,%d = %s\n", gloc->pos_h, gloc->pos_v, gloc->name);
}

char *
gridloc_getname (GridLoc *gloc)
{
  return gloc->name;
}

GridLoc *
grid_get (Grid *grid, GridLoc *gloc, int dir)
{
  int     i;

  switch (dir) {
    case SCREEN_LEFT:
      for (i = gloc->pos_h - 1; i >= gloc->pos_h - GRID_SIZE_COL; i--)
        if (grid->grid[gloc->pos_v][(i < 0 ? i + GRID_SIZE_COL : i) % GRID_SIZE_COL] != 0)
          return (GridLoc *)grid->grid[gloc->pos_v][(i < 0 ? i + GRID_SIZE_COL : i) % GRID_SIZE_COL];
      break;
    case SCREEN_RIGHT:
      for (i = gloc->pos_h + 1; i <= GRID_SIZE_COL; i++)
        if (grid->grid[gloc->pos_v][i % (GRID_SIZE_COL - 1)] != 0)
          return (GridLoc *)grid->grid[gloc->pos_v][i % (GRID_SIZE_COL - 1)];
      break;
    case SCREEN_UP:
      for (i = gloc->pos_v - 1; i >= gloc->pos_v - GRID_SIZE_ROW; i--)
        if (grid->grid[(i < 0 ? i + GRID_SIZE_COL : i)][gloc->pos_h] != 0)
          return (GridLoc *)grid->grid[(i < 0 ? i + GRID_SIZE_COL : i)][gloc->pos_h];
      break;
    case SCREEN_DOWN:
      for (i = gloc->pos_v + 1; i <= GRID_SIZE_ROW; i++)
        if (grid->grid[i % (GRID_SIZE_ROW - 1)][gloc->pos_h] != 0)
          return (GridLoc *)grid->grid[i % (GRID_SIZE_ROW - 1)][gloc->pos_h];
      break;
    default:
      fprintf (stderr, "grid_get: invalid directional move: %d\n", dir);
  }

  return NULL;
}

GridLoc *
grid_getbyname (Grid *grid, char *name)
{
  int i, j;

  for (i = 0; i < GRID_SIZE_COL; i++)
    for (j = 0; j < GRID_SIZE_ROW; j++)
      if (grid->grid[j][i] != 0)
        if (0 == strcmp (((GridLoc *)grid->grid[j][i])->name, name))
          return (GridLoc *)grid->grid[j][i];

  return NULL;
}

void
grid_del (Grid *grid, GridLoc *gloc)
{
  int i, j;

  printf ("grid_del: deleting gridloc->name = %s\n", gloc->name);

  for (i = 0; i < GRID_SIZE_COL; i++)
    for (j = 0; j < GRID_SIZE_ROW; j++)
      if (grid->grid[j][i] != 0)
        if (0 == strcmp (((GridLoc *)grid->grid[j][i])->name, gloc->name)) {
          grid->grid[j][i] = 0;
	  printf ("grid_del: deleted %s from %.2d x %.2d\n", gloc->name, i, j);
        }
}

