#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h> 

char *
gx11_proc_clipboard (display, mainwin, cb_src, cb_src_len)
  Display *display;
  Window mainwin;
  char *cb_src;
  int cb_src_len;
{
  char *cb_dst;

  XSetSelectionOwner (display, XA_PRIMARY, mainwin, CurrentTime);

  if (NULL == (cb_dst = malloc (cb_src_len))) {
    fprintf (stderr, "gx11_proc_clipboard: failed to malloc()\n");
    return NULL;
  }

  memcpy (cb_dst, cb_src, cb_src_len);

  return cb_dst;
}

void
gx11_proc_selection_send (clipboard, clipboard_len, sre)
  char *clipboard;
  int clipboard_len;
  XSelectionRequestEvent sre;
{
  XEvent     xnotify;

  XChangeProperty (sre.display, sre.requestor, sre.property, sre.target, 8,
  PropModeReplace, clipboard, clipboard_len);

  xnotify.xselection.type = SelectionNotify;
  xnotify.xselection.property = sre.property;
  xnotify.xselection.display = sre.display;
  xnotify.xselection.requestor = sre.requestor;
  xnotify.xselection.selection = sre.selection;
  xnotify.xselection.target = sre.target;
  xnotify.xselection.time = sre.time;

  XSendEvent (sre.display, sre.requestor, False, 0, &xnotify);
  XFlush (sre.display);
}

char *
gx11_proc_selection_notify (display, window, cb_len)
  Display *display;
  Window window;
  int *cb_len;
{
  long nread;
  unsigned long bytes_after, nitems;
  unsigned char *data;
  Atom actual_type;
  int actual_fmt;
  char *dst = NULL;

  *cb_len = 0;

  for (nread = 0, bytes_after = 1; bytes_after > 0; ) {
    if ((XGetWindowProperty (display, window, XA_CUT_BUFFER0, 0L, 4096,
                             False, AnyPropertyType, &actual_type, &actual_fmt, &nitems, &bytes_after,
                             &data) != Success) || (actual_type == None) || (data == NULL)) {
      if (data != NULL)
        XFree (data);

      break;
    }
    else {
      nread += nitems;
      if (nitems == 0)
        break;

      if (actual_type == XA_STRING) {
        if (NULL == (dst = realloc (dst, nread))) {
          fprintf (stderr, "gx11_proc_selection_notify: failed to realloc()\n");
          free (dst);
          return NULL;
        }

        strncpy (dst + (nread-nitems), data, nitems);

        if (bytes_after == 0)
          break;
      }
      else
        printf ("gx11_proc_motion: unknown data type\n");
    }
  }

  *cb_len = nread;

  return dst;
}

void
gx11_pointer_pos_get (display, ret_x, ret_y)
  Display *display;
  int *ret_x;
  int *ret_y;
{
  Window     window;
  XEvent     xevent;

  window = DefaultRootWindow (display);

  XQueryPointer (display, window, &xevent.xbutton.root, &xevent.xbutton.window,
                 &xevent.xbutton.x_root, &xevent.xbutton.y_root, &xevent.xbutton.x,
		 &xevent.xbutton.y, &xevent.xbutton.state);

  *ret_x = xevent.xbutton.x;
  *ret_y = xevent.xbutton.y;
}

int
gx11_pointer_state_get (display)
  Display *display;
{
  Window     window;
  XEvent     xevent;

  window = DefaultRootWindow (display);

  XQueryPointer (display, window, &xevent.xbutton.root, &xevent.xbutton.window,
                 &xevent.xbutton.x_root, &xevent.xbutton.y_root, &xevent.xbutton.x,
		 &xevent.xbutton.y, &xevent.xbutton.state);

  return xevent.xbutton.state;
}

