#ifndef __NETWORK_H
#define __NETWORK_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "xgroove.h"

typedef struct _UDPQuery {
  char      *name;            /* name of master desired    */
} UDPQuery;

typedef struct _UDPQueryResponse {
  int       resp;             /* 1 == yes; 0 == no         */
  int       port;             /* tcp port to report back   */
  char      *addr;            /* ip address to report back */
  char      *name;            /* this master's name        */
} UDPQueryResponse;

typedef struct _UDPConn {
  int                      port;
  int                      sock;
  struct sockaddr_in       sock_info;
  char                     *buf;
  int                      bytes_in;       /* total bytes read from socket  */
  int                      bytes_out;      /* total bytes written to socket */
  int                      so_timeout;
} UDPConn;

typedef struct _TCPLstn {
  int                      port;
  int                      sock;
  char                     *addr;
  struct sockaddr_in       sock_info;
} TCPLstn;

typedef struct _TCPConn {
  int                      port;           /* local port, bound to          */
  char                     *addr;          /* local interface, bound to     */
  int                      sock;           /* file description for socket   */
  struct sockaddr_in       *sock_info;     /* socket info stuff             */
  int                      bytes_in;       /* total bytes read from socket  */
  int                      bytes_out;      /* total bytes written to socket */
  char                     *buf;           /* socket read buffer            */
  int                      buf_leng;       /* socket read buffer length     */
  int                      is_connected;   /* 1 = yes, otherwise = no       */
  int                      so_timeout;
} TCPConn;

typedef struct _UDPMesg {
  struct sockaddr_in       info;           /* protocol address info         */
  socklen_t                info_len;       /* length of protocal address    */
  int                      port;           /* outgoing port of sender       */
  char                     addr[16];       /* sender's ip address           */
  GEvent                   *gevent;
  UDPConn                  *conn;
  int                      is_reply;       /* 1 = yes; otherwise, no        */
} UDPMesg;

UDPConn * udp_open_connection (char *addr, int port);     /* create udp 'connection' */
void udp_close_connection (UDPConn *conn);                /* close udp 'connection'  */

TCPLstn * tcp_open_listener (int port);                   /* create tcp listener     */
TCPConn * tcp_open_connection (char *addr, int port);     /* create tcp connection   */
void tcp_close_listener (TCPLstn *lstn);                  /* close tcp listener      */
void tcp_close_connection (TCPConn *conn);                /* close tcp connection    */

int udp_send (UDPConn *conn, UDPMesg  *mesg, char *addr, int port);
int udp_recv (UDPConn *conn, UDPMesg  *mesg);
int tcp_recv (TCPConn *conn, GEvent *gevent);
int tcp_send (TCPConn *conn, GEvent *gevent);

void udp_set_sotimeout (UDPConn *conn, int sotimeout);
void tcp_set_sotimeout (TCPConn *conn, int sotimeout);

TCPConn * tcp_accept (TCPLstn *lstn);

#endif /* __NETWORK_H */

