#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <pthread.h>
#include <getopt.h>
#include <string.h>

#include "xgroove.h"
#include "grid.h"
#include "network.h"
/* #include "xgmaster.h" */
#include "xgslave.h"

int
main (int argc, char *argv[])
{
  if (argc != 3) {
    fprintf (stderr, "usage: %s [master|slave] [name]\n", argv[0]);
    exit (1);
  }

/*
  if (0 == strcmp (argv[1], "master") ) {
    do_gmaster (argc, argv);
  }
  else if (0 == strcmp (argv[1], "slave") ) {
    do_gslave (argc, argv);
  }
*/

  if (0 == strcmp (argv[1] == "master")) {
    fprintf (stderr, "main: mode not supported on this platform: master\n");
    exit (1);
  }
  else if (0 == strcmp (argv[1] = "slave")) {
    do_gslave (argc, argv);
  }
  else {
    fprintf (stderr, "usage: %s [master|slave] [name]\n", argv[0]);
    exit (1);
  }

  return 0;
}

GEvent *
init_gevent ()
{
  GEvent *gevent;

  if (NULL == (gevent = malloc (sizeof(GEvent))) ) {
    fprintf (stderr, "init_gevent: failed to malloc()\n");
    return NULL;
  }

  bzero (gevent, sizeof(GEvent));

  if (NULL == (gevent->generic.serial = malloc (sizeof(char) * 8)) ) {
    fprintf (stderr, "gevent_serialize: failed in realloc() [1]\n");
    return NULL;
  }

  memcpy (gevent->generic.serial, "0000000:", 8);

  gevent->generic.type = 0;
  gevent->generic.serial_len = 8;
  gevent->generic.serial_mem = 8;
  gevent->generic.is_inited = 1;
  gevent->generic.has_been_used = 0;
  gevent->generic.chunk_num = 1;
  gevent->generic.chunk_tot = 1;

  return gevent;
}

void
deinit_gevent (GEvent *gevent)
{
  if (gevent->generic.serial != NULL && gevent->generic.serial_mem > 0)
    free (gevent->generic.serial);

  gevent->generic.serial_len = 0;
  gevent->generic.serial_mem = 0;

  switch (gevent->generic.type) {
  case GrooveClipboardEvent:
    if (gevent->gclipboard.clipboard != NULL && gevent->gclipboard.clipboard_len > 0) {
      free (gevent->gclipboard.clipboard);
      gevent->gclipboard.clipboard = NULL;
      gevent->gclipboard.clipboard_len = 0;
    }
    break;
  default:
    break;
  }

  free (gevent);
}

int
gevent_serialize (GEvent *gevent)
{
  char     buf[16];
  int      size;

  if (gevent->generic.is_inited != 1) {
    fprintf (stderr, "gevent_serialize: please run init_gevent() first\n");
    return -1;
  }

  if (gevent->generic.type < 1) {
    fprintf (stderr, "gevent_serialize: invalid gevent type: %d\n", gevent->type);
    return -1;
  }

  /* printf ("gevent_serialize: processing type: %d\n", gevent->generic.type); */
  gevent->generic.serial_len = 8;
  memcpy (gevent->generic.serial, "0000000:", 8);

  gevent_serialize_num (gevent, "type", gevent->generic.type);

  switch (gevent->generic.type) {
    case GrooveMotionEvent:
      gevent_serialize_num (gevent, "motion_type", gevent->gmotion.motion_type);
      gevent_serialize_num (gevent, "x", gevent->gmotion.x);
      gevent_serialize_num (gevent, "y", gevent->gmotion.y);
      break;
    case GrooveClickEvent:
      gevent_serialize_num (gevent, "click_type", gevent->gclick.click_type);
      gevent_serialize_num (gevent, "button", gevent->gclick.button);
   /* gevent_serialize_num (gevent, "x", gevent->gclick.x); */
   /* gevent_serialize_num (gevent, "y", gevent->gclick.y); */
      break;
    case GrooveKeyPressEvent:
      gevent_serialize_num (gevent, "keypress_type", gevent->gkeypress.keypress_type);
      gevent_serialize_num (gevent, "keycode", gevent->gkeypress.keycode);
      break;
    case GrooveControlEvent:
      gevent_serialize_num (gevent, "control_type", gevent->gcontrol.control_type);
      break;
    case GrooveHelloEvent:
      gevent_serialize_num (gevent, "disp_width", gevent->ghello.disp_width);
      gevent_serialize_num (gevent, "disp_height", gevent->ghello.disp_height);
      gevent_serialize_num (gevent, "port", gevent->ghello.port);
      gevent_serialize_str (gevent, "name", gevent->ghello.name, -1);
      gevent_serialize_num (gevent, "machine_type", gevent->ghello.machine_type);
      break;
    case GrooveHelloBackEvent:
      gevent_serialize_str (gevent, "addr", gevent->ghelloback.addr, -1);
      gevent_serialize_num (gevent, "port", gevent->ghelloback.port);
      break;
    case GrooveGoodbyeEvent:
      break;
    case GrooveClipboardEvent:
      gevent_serialize_str (gevent, "clipboard", gevent->gclipboard.clipboard, gevent->gclipboard.clipboard_len);
      break;
    case GrooveConfigureEvent:
      gevent_serialize_num (gevent, "width", gevent->gconfigure.width);
      gevent_serialize_num (gevent, "height", gevent->gconfigure.height);
      break;
    default:
      fprintf (stderr, "gevent_serialize: invalid gevent type: %d\n", gevent->generic.type);
      return -1;
  }

  size = snprintf (buf, 16, "%d", gevent->generic.serial_len - 8);
  memcpy (&gevent->generic.serial[7 - size], buf, size);

  /* printf ("gevent_serialize: [%d] %s\n", gevent->generic.serial_len, gevent->generic.serial); */

  return gevent->generic.serial_len;
}

int
gevent_serialize_str (gevent, name, data, data_len)
  GEvent *gevent;
  char *name;
  char *data;
  int data_len;
{
  char     len_buf[16];
  int      len_len, str_tot;
  int      name_len = strlen (name);

  if (gevent->generic.is_inited != 1) {
    fprintf (stderr, "gevent_doit: please run init_gevent() first\n");
    return -1;
  }

  if (data_len == -1)
    data_len = strlen (data);

  len_len = snprintf (len_buf, 16, "%d", data_len);
  str_tot = gevent->generic.serial_len + name_len + data_len + len_len + 3;

  if (gevent->generic.serial_mem < str_tot) {
    if (NULL == (gevent->generic.serial = realloc (gevent->generic.serial, str_tot)) ) {
      fprintf (stderr, "gevent_doit: failed in malloc()\n");
      return -1;
    }
    gevent->generic.serial_mem = str_tot;
  }

  /* copy the value name */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, name, name_len);
  gevent->generic.serial_len += name_len;

  /* memcpy (gevent->generic.serial + gevent->generic.serial_len, ":", 1); */
  /* gevent->generic.serial_len += 1; */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  /* copy the value length */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, len_buf, len_len);
  gevent->generic.serial_len += len_len;

  /* memcpy (gevent->generic.serial + gevent->generic.serial_len, ":", 1); */
  /* gevent->generic.serial_len += 1; */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  /* copy the value */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, data, data_len);
  gevent->generic.serial_len += data_len;

  /* append a string terminator */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  return str_tot - gevent->generic.serial_len;
}

int
gevent_serialize_num (GEvent *gevent, char *name, int value)
{
  char     val_buf[16], len_buf[16];
  int      val_len, len_len, str_tot;
  int      name_len = strlen (name);

  if (gevent->generic.is_inited != 1) {
    fprintf (stderr, "gevent_doit: please run init_gevent() first\n");
    return -1;
  }

  val_len = snprintf (val_buf, 16, "%d", value);     /* the length of the value in ascii */
  len_len = snprintf (len_buf, 16, "%d", val_len);   /* the length of the value, that's the length of the value */
  str_tot = gevent->generic.serial_len + name_len + val_len + len_len + 3;

  if (gevent->generic.serial_mem < str_tot) {
    if (NULL == (gevent->generic.serial = realloc (gevent->generic.serial, str_tot)) ) {
      fprintf (stderr, "gevent_doit: failed in malloc()\n");
      return -1;
    }
    gevent->generic.serial_mem = str_tot;
  }

  /* copy the value name */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, name, name_len);
  gevent->generic.serial_len += name_len;

  /* memcpy (gevent->generic.serial + gevent->generic.serial_len, ":", 1); */
  /* gevent->generic.serial_len += 1; */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  /* copy the value length */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, len_buf, len_len);
  gevent->generic.serial_len += len_len;

  /* memcpy (gevent->generic.serial + gevent->generic.serial_len, ":", 1); */
  /* gevent->generic.serial_len += 1; */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  /* copy the value */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, val_buf, val_len);
  gevent->generic.serial_len += val_len;

  /* append a string terminator */
  memcpy (gevent->generic.serial + gevent->generic.serial_len, "\0", 1);
  gevent->generic.serial_len += 1;

  return str_tot - gevent->generic.serial_len;
}

GEvent *
gevent_deserialize (gevent, data, data_len)
  GEvent *gevent;
  char *data;
  int data_len;
{
  int     pos1, pos2, len; 
        
  if (gevent->generic.is_inited != 1) {
    fprintf (stderr, "gevent_deserialize: please run init_gevent() first\n");
    return NULL;
  }

  if (gevent->has_been_used == 1) {
    fprintf (stderr, "gevent_deserialize: has already been used * * *\n");
    return NULL;
  }

  gevent->has_been_used = 1;

  data += 8;
  data_len -= 8;

  for (pos1 = 0; pos1 < data_len; pos1++) {
    if (data[pos1] != '\0')
      continue;

    for (pos2 = pos1 + 1; pos2 < data_len; pos2++) {
      if (data[pos2] != '\0')
        continue;

      len = atoi (data + pos1 + 1);

      gevent_deserialize_doit (gevent, data, pos1, &data[pos2+1], len);

      data += pos2 + len + 2;
      data_len -= (pos2 + len + 2);
      pos1 = 0;

      break;
    }
  }

  return gevent;
}

void
gevent_deserialize_doit (gevent, name, name_len, data, data_len)
  GEvent *gevent;
  char *name;
  int name_len;
  char *data;
  int data_len;
{
  if (gevent->generic.is_inited != 1) {
    fprintf (stderr, "gevent_deserialize_doit: please run init_gevent() first\n");
    return;
  }

  if (0 == strcmp (name, "type")) {
    gevent->generic.type = atoi (data);
    return;
  }

  switch (gevent->generic.type) {
    case GrooveMotionEvent:
      if (0 == strcmp (name, "motion_type"))
        gevent->gmotion.motion_type = atoi (data);
      else if (0 == strcmp (name, "x"))
        gevent->gmotion.x = atoi (data);
      else if (0 == strcmp (name, "y"))
        gevent->gmotion.y = atoi (data);
      break;
    case GrooveClickEvent:
      if (0 == strcmp (name, "click_type"))
        gevent->gclick.click_type = atoi (data);
      else if (0 == strcmp (name, "button"))
        gevent->gclick.button = atoi (data);
      break;
    case GrooveKeyPressEvent:
      if (0 == strcmp (name, "keypress_type"))
        gevent->gkeypress.keypress_type = atoi (data);
      else if (0 == strcmp (name, "keycode"))
        gevent->gkeypress.keycode = atoi (data);
      break;
    case GrooveControlEvent:
      if (0 == strcmp (name, "control_type"))
        gevent->gcontrol.control_type = atoi (data);
    case GrooveHelloEvent:
      if (0 == strcmp (name, "disp_width"))
        gevent->ghello.disp_width = atoi (data);
      else if (0 == strcmp (name, "disp_height"))
        gevent->ghello.disp_height = atoi (data);
      else if (0 == strcmp (name, "port"))
        gevent->ghello.port = atoi (data);
      else if (0 == strcmp (name, "name"))
        memcpy (gevent->ghello.name, data, min (data_len, sizeof (gevent->ghello.name) ));
      else if (0 == strcmp (name, "machine_type"))
        gevent->ghello.machine_type = atoi (data);
      break;
    case GrooveHelloBackEvent:
      if (0 == strcmp (name, "port"))
        gevent->ghelloback.port = atoi (data);
      else if (0 == strcmp (name, "addr"))
        memcpy (gevent->ghelloback.addr, data, min (data_len, sizeof (gevent->ghelloback.addr) ));
      break;
    case GrooveGoodbyeEvent:
      break;
    case GrooveClipboardEvent:
      if (0 == strcmp (name, "clipboard")) {
/*
 *      if (gevent->gclipboard.clipboard != NULL && gevent->gclipboard.clipboard_len > 0) {
 *        printf ("gevent_deserialize_doit: removing old clipboard memory.\n");
 *        free (gevent->gclipboard.clipboard);
 *        gevent->gclipboard.clipboard = NULL;
 *        gevent->gclipboard.clipboard_len = 0;
 *      }
 */
        if (NULL == (gevent->gclipboard.clipboard = malloc (data_len))) {
	  fprintf (stderr, "gevent_deserialize_doit: failed in malloc(%d)\n", data_len);
	  return;
	}
	gevent->gclipboard.clipboard_len = data_len;
        memcpy (gevent->gclipboard.clipboard, data, data_len);
      }
      break;
    case GrooveConfigureEvent:
      if (0 == strcmp (name, "width"))
        gevent->gconfigure.width = atoi (data);
      else if (0 == strcmp (name, "height"))
        gevent->gconfigure.height = atoi (data);
      break;
    case GrooveScreenSaverEvent:
      if (0 == strcmp (name, "ss_type"))
        gevent->gscreensaver.ss_type = atoi (data);
      break;
    default:
      break;
  }
}

