#ifndef __XGROOVE_H
#define __XGROOVE_H

#include <sys/types.h>
#include <sys/socket.h>
#include <pthread.h>

#include <X11/Xlib.h>

#define DEFAULT_TCP_PORT 8501
#define DEFAULT_UDP_PORT 8502

#define GrooveMotionEvent       1
#define GrooveClickEvent        2
#define GrooveKeyPressEvent     3
#define GrooveControlEvent      4
#define GrooveHelloEvent        5
#define GrooveHelloBackEvent    6
#define GrooveGoodbyeEvent      7
#define GrooveClipboardEvent    8
#define GrooveConfigureEvent    9
#define GrooveScreenSaverEvent 10

#define GrooveMotionTypeAbsolute 1
#define GrooveMotionTypeRelative 2

#define GrooveControlTypeSeize  1
#define GrooveControlTypeCease  2
#define GrooveControlTypeSiezed 3
#define GrooveControlTypeCeased 4

#define GrooveClickTypePress   1
#define GrooveClickTypeRelease 2

#define GrooveClickButton1 1
#define GrooveClickButton2 2
#define GrooveClickButton3 3
#define GrooveClickButton4 4
#define GrooveClickButton5 5

#define GrooveKeyPressTypePress   1
#define GrooveKeyPressTypeRelease 2

#define GrooveScreenSaverTypeLock   1
#define GrooveScreenSaverTypeUnlock 2

#define GrooveMachineTypeX11 1
#define GrooveMachineTypeOSX 2
#define GrooveMachineTypeWIN 3

#define SCREEN_LEFT  1
#define SCREEN_RIGHT 2
#define SCREEN_UP    3
#define SCREEN_DOWN  4

#define max(A,B) ((A) > (B) ? (A) : (B))
#define min(A,B) ((A) < (B) ? (A) : (B))

typedef struct _GDisplay {
  int        width;                    /* display pixel width         */
  int        height;                   /* display pixel height        */
  int        pos_h;                    /* pointer position on screen  */
  int        pos_v;                    /* pointer position on screen  */
  Display    *display;                 /* x display object (if slave) */
  Window     *rootwin;                 /* root x window (if slave)    */
  Window     mainwin;                  /* main, invisible window      */
} GDisplay;

typedef struct _GMotionEvent {
  int             type;                /* GrooveMotionEvent           */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             motion_type;         /* absolute or relative        */
  int             x;                   /* horizontal position/motion  */
  int             y;                   /* vertical position/motion    */
} GMotionEvent;

typedef struct _GClickEvent {
  int             type;                /* GrooveClickEvent            */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             click_type;          /* press or release            */
  int             button;              /* GrooveClickButtonX          */
  int             x;                   /* horizontal pixel location   */
  int             y;                   /* vertical pixel location     */
} GClickEvent;

typedef struct _GKeyPressEvent {
  int             type;                /* GrooveKeyPressEvent         */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             keypress_type;       /* press or release            */
  int             keycode;             /* the keycode to type         */
} GKeyPressEvent;

typedef struct _GControlEvent {
  int             type;                /* GrooveControlEvent          */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             control_type;        /* seize or cease              */
} GControlEvent;

typedef struct _GHelloEvent {
  int             type;                /* GrooveHelloEvent            */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             disp_width;          /* width (pixels) of display   */
  int             disp_height;         /* height (pixels) of display  */
  int             port;                /* port on which to reply      */
  char            name[16];            /* name of desired server      */
  int             machine_type;        /* GrooveMachineTypeX          */
} GHelloEvent;

typedef struct _GHelloBackEvent {
  int             type;                /* GrooveHelloEvent            */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  char            addr[32];            /* ip address of server        */
  int             port;                /* port of tcp listener        */
} GHelloBackEvent;

typedef struct _GGoodbyeEvent {
  int             type;                /* GrooveHelloEvent            */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
} GGoodbyeEvent;

typedef struct _GClipboardEvent {
  int             type;                /* GrooveHelloEvent            */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  char            *clipboard;          /* clipboard data              */
  int             clipboard_len;       /* clipboard data length       */
} GClipboardEvent;

typedef struct _GUnknownEvent {
  int             type;                /* generic GrooveEventX        */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
} GUnknownEvent;

typedef struct _GConfigureEvent {
  int             type;                /* generic GrooveEventX        */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             width;               /* remote display width        */
  int             height;              /* remove display height       */
} GConfigureEvent;

typedef struct _GScreenSaverEvent {
  int             type;                /* generic GrooveEventX        */
  char            *serial;             /* serialized event data       */
  int             serial_len;          /* length of serialized data   */
  int             serial_mem;          /* malloc length of *serial    */
  int             is_inited;           /* 1 = yes, otherwise = no     */
  int             has_been_used;       /* 1 = yes, otherwise = no     */
  int             chunk_num;           /* message chunck number       */
  int             chunk_tot;           /* total num of message chunks */
  int             ss_type;             /* lock or unlock              */
} GScreenSaverEvent;

typedef union _GEvent {
  int               type;                /* generic GrooveEventX        */
  char              *serial;             /* serialized event data       */
  int               serial_len;          /* length of serialized data   */
  int               serial_mem;          /* malloc length of *serial    */
  int               is_inited;           /* 1 = yes, otherwise = no     */
  int               has_been_used;       /* 1 = yes, otherwise = no     */
  int               chunk_num;           /* message chunck number       */
  int               chunk_tot;           /* total num of message chunks */
  GUnknownEvent     generic;             /* for being a bad programmer  */
  GMotionEvent      gmotion;             /* do something with the mouse */
  GClickEvent       gclick;              /* click the mouse             */
  GKeyPressEvent    gkeypress;           /* press/release a key         */
  GControlEvent     gcontrol;            /* take/release control        */
  GHelloEvent       ghello;              /* say hello, init connection  */
  GHelloBackEvent   ghelloback;          /* server response to a hello  */
  GGoodbyeEvent     ggoodbye;            /* close connection event      */
  GClipboardEvent   gclipboard;          /* clipboard copy event        */
  GConfigureEvent   gconfigure;          /* slave screen information    */
  GScreenSaverEvent gscreensaver;        /* un/lock the screen saver    */
} GEvent;

GEvent * init_gevent ();
void deinit_gevent (GEvent *gevent);
int gevent_serialize (GEvent *gevent);
int gevent_serialize_num (GEvent *gevent, char *name, int value);
int gevent_serialize_str (GEvent *gevent, char *name, char *data, int data_len);
GEvent * gevent_deserialize (GEvent *gevent, char *data, int data_len);
void gevent_deserialize_doit (GEvent *gevent, char *name, int name_len, char *data, int data_len);

#endif /* __XGROOVE_H */

