#ifndef __XGSLAVE_H
#define __XGSLAVE_H

#include <sys/types.h>
#include <sys/socket.h>
#include <pthread.h>

#include "xgroove.h"
#include "network.h"

typedef struct _GSlave {
  int        socket;
  char       name[16];            /* remote host's target name                     */
  char       *addr;               /* remote host ip address                        */
  int        port;                /* remote host port number                       */
  pthread_t  thread;
  GDisplay   gdisplay;            /* display information                           */
  TCPConn    *tcp_conn;           /* tcp connection to the master                  */
  UDPConn    *udp_conn;           /* for hooking up with a master                  */
  int        has_control;         /* 1 = yes, otherwise = no                       */
  int        is_pressed;          /* is the mouse pressed?                         */
  int        prev_pos_h;          /* the loc of the mouse, prior to taking control */
  int        prev_pos_v;          /* the loc of the mouse, prior to taking control */
  pthread_t  xevent_thread;       /* processes any xevents we receive              */
  char       *clipboard;
  int        clipboard_len;
  void       *gmaster;
  int        is_controlled;
} GSlave;

GSlave * init_gslave (char *name);
void deinit_gslave (GSlave *gslave);

void gslave_loop (GSlave *gslave);
int gslave_find_master (GSlave *gslave);
void do_gslave (int argc, char *argv[]);
void gslave_xevent_loop (GSlave *gslave);
void gslave_xevent_selection_send (GSlave *gslave, XSelectionRequestEvent sre);
void gslave_proc_selection_notify (GSlave *gslave, Window window);

void gslave_proc (GSlave *gslave, GEvent *gevent);
void gslave_proc_motion (GSlave *gslave, GEvent *gevent);
void gslave_proc_click (GSlave *gslave, GEvent *gevent);
void gslave_proc_keypress (GSlave *gslave, GEvent *gevent);
void gslave_proc_control (GSlave *gslave, GEvent *gevent);
void gslave_proc_hello (GSlave *gslave, GEvent *gevent);
void gslave_proc_helloback (GSlave *gslave, GEvent *gevent);
void gslave_proc_goodbye (GSlave *gslave, GEvent *gevent);
void gslave_proc_clipboard (GSlave *gslave, GClipboardEvent gce);

#endif /* __XGSLAVE_H */

